@@ -4,7 +4,7 @@ use 5.008_001;
 use strict;
 use warnings;
 
-our $VERSION = '0.1058';
+our $VERSION = '0.2000';
 
 use Carp        ();
 use File::Spec  ();
@@ -17,7 +17,7 @@ use Text::Xslate::Util qw(
     uri_escape html_builder
 );
 
-our @ISA = qw(Text::Xslate::Engine Exporter);
+our @ISA = qw(Text::Xslate::Engine);
 
 our @EXPORT_OK = qw(
     mark_raw
@@ -39,7 +39,6 @@ if(!__PACKAGE__->can('render')) { # The backend is already loaded
     }
     if(!__PACKAGE__->can('render')) { # failed to load XS, or force PP
         require 'Text/Xslate/PP.pm';
-        Text::Xslate::PP->import(':backend');
     }
 }
 
@@ -53,6 +52,8 @@ use Text::Xslate::Util qw(
 );
 
 BEGIN {
+    our @ISA = qw(Exporter);
+
     my $dump_load = scalar($DEBUG =~ /\b dump=load \b/xms);
     *_DUMP_LOAD = sub(){ $dump_load };
 
@@ -83,7 +84,7 @@ my %parser_option = (
 # the real defaults are defined in the compiler
 my %compiler_option = (
     syntax     => undef,
-    escape     => undef,
+    type       => undef,
     header     => undef,
     footer     => undef,
 );
@@ -314,7 +315,8 @@ sub _load_source {
         my $cachedir      = File::Spec->catpath($volume, $dir, '');
         if(not -e $cachedir) {
             require File::Path;
-            File::Path::mkpath($cachedir);
+            eval { File::Path::mkpath($cachedir) }
+                or Carp::carp("Xslate: Cannot make directory $cachepath (ignored): $@");
         }
 
         # use input_layer for caches
@@ -503,11 +505,11 @@ __END__
 
 =head1 NAME
 
-Text::Xslate - High performance template engine
+Text::Xslate - Scalable template engine for Perl5
 
 =head1 VERSION
 
-This document describes Text::Xslate version 0.1058.
+This document describes Text::Xslate version 0.2000.
 
 =head1 SYNOPSIS
 
@@ -547,21 +549,16 @@ This document describes Text::Xslate version 0.1058.
 
 =head1 DESCRIPTION
 
-B<Text::Xslate> is a high performance template engine tuned for persistent
-applications.
+B<Text::Xslate> is a template engine tuned for persistent applications.
 This engine introduces the virtual machine paradigm. Templates are
-compiled into xslate intermediate code, and then executed by the xslate
-virtual machine.
+compiled into intermediate code, and then executed by the virtual machine.
 
 The concept of Xslate is strongly influenced by Text::MicroTemplate
-and Template-Toolkit, but the central philosophy of Xslate is different from them.
+and Template-Toolkit 2, but the central philosophy of Xslate is different
+from them.
 That is, the philosophy is B<sandboxing> that the template logic should
 not have no access outside the template beyond your permission.
 
-B<This software is under development>.
-Version 0.1xxx is a developing stage, which may include radical changes.
-Version 0.2xxx and more will be somewhat stable.
-
 =head2 Features
 
 =head3 High performance
@@ -658,7 +655,8 @@ You B<should> specify this option on productions.
 
 =item C<< function => \%functions >>
 
-Specifies a function map. A function C<f> may be called as C<f($arg)> or C<$arg | f>.
+Specifies a function map which contains name-coderef pairs.
+A function C<f> may be called as C<f($arg)> or C<$arg | f> in templates.
 
 There are a few builtin filters, but they are not overridable.
 
@@ -693,7 +691,7 @@ specific namespaces for plugins.
 
 =item C<< input_layer => $perliolayers // ':utf8' >>
 
-Specifies PerlIO layers for reading templates.
+Specifies PerlIO layers to open template files.
 
 =item C<< verbose => $level // 1 >>
 
@@ -719,14 +717,13 @@ I<$name> may be a short name (e.g. C<Kolon>), or a fully qualified name
 
 This option is passed to the compiler directly.
 
-=item C<< escape => $mode // 'html' >>
+=item C<< type => $type // 'html' >>
 
-Specifies the escape mode, which is automatically applied to template expressions.
+Specifies the output content type. If I<$type> is C<html> or C<xml>,
+template expressions are interpolated via the C<html-escape> filter.
+If I<$type> is C<text>, template expressions are interpolated as they are.
 
-Possible escape modes are B<html> and B<none>.
-
-Note that C<none> mode is provided for non-HTML templates, e.g. mail generators,
-so you must not to use it for HTML templates because it is unsafe.
+I<$type> may be B<html>, B<xml> (identical to C<html>), and B<text>.
 
 This option is passed to the compiler directly.
 
@@ -774,7 +771,8 @@ Note that I<$file> may be cached according to the cache level.
 Renders a template string with variables, and returns the result.
 I<\%vars> is optional.
 
-Note that I<$string> is never cached, so this may be not suitable for web applications.
+Note that I<$string> is never cached, so this method may not be suitable for
+productions.
 
 =head3 B<< $tx->load_file($file) :Void >>
 
@@ -1008,6 +1006,8 @@ Benchmarks:
 
 L<Template::Benchmark>
 
+L<http://xslate.org/benchmark.html>
+
 =head1 ACKNOWLEDGEMENT
 
 Thanks to lestrrat for the suggestion to the interface of C<render()>,
@@ -1029,6 +1029,8 @@ Thanks to punytan for the documentation improvement.
 
 Thanks to chiba for the bug reports and patches.
 
+Thanks to turugina for the patch to fix Win32 problems
+
 =head1 AUTHOR
 
 Fuji, Goro (gfx) E<lt>gfuji(at)cpan.orgE<gt>
